<?php
class AlphasmsAdmin
{
	protected $page;
	protected $capability;
	protected $url;
	protected $functions_page;
	protected $view_path;
	protected $config;
	
	public $settings = [];
	public $is_plugin_page = 0;
	
	public function __construct($is_plugin_page)
	{
		$this->page = [
			1=>__('AlphaSMS', 'alphasms'),
		];
		$this->capability = 'edit_others_pages';
		$this->url = [
			1=>'alphasms-sms',
		];
		$this->functions_page = [
			1=>'settings_page_sms',
		];
		
		$this->is_plugin_page = $is_plugin_page;
		$this->view_path = realpath(ALPHASMS_DIR) . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR;
	}
	
	/**
	 * wpAdmin()
	 */
	public function wpAdmin()
	{
		add_action('admin_menu', [$this, 'settings_menu']);
		add_action('wp_ajax_alphasms_balance_action', [$this, 'balance_action']);
		add_action('wp_ajax_alphasms_select_template', [$this, 'select_template']);
		add_action('wp_ajax_alphasms_load_form_smssettings', [$this, 'load_form_smssettings']);
		add_action('wp_ajax_alphasms_load_form_smsmailing', [$this, 'load_form_smsmailing']);
		add_action('wp_ajax_alphasms_load_form_smsevents', [$this, 'load_form_smsevents']);
		add_action('wp_ajax_alphasms_load_form_smstemplates', [$this, 'load_form_smstemplates']);
	}

	/**
	 * settings_menu()
	 */
	public function settings_menu()
	{
		add_submenu_page(
			'woocommerce',
			$this->page[1],
			$this->page[1], 
			$this->capability, 
			$this->url[1],
			[$this, $this->functions_page[1]] 
		);
	}

	/**
	 * settings_page_sms()
	 */
	public function settings_page_sms()
	{
		if (!empty($this->is_plugin_page)) {
			
			self::resourceRegistration();
			
			$this->settings = $this->settings + AlphasmsDB::alphasms()->loadSettings('sms');
			$this->settings['error'] = false;
		}

		if (file_exists($this->view_path.'settings_page_sms.php')) {
			require_once $this->view_path.'settings_page_sms.php';
		}
	}

	/**
	 * balance_action() 
	 */
	public function balance_action() 
	{
		$balance = 0;
		if (!self::validateToken($_POST['token'])) {
			echo $balance;
			wp_die(); 
		}

		$settings = AlphasmsDB::alphasms()->loadSettings('sms');

		if (!empty($_POST['type']) && $_POST['type']==='balance') {
			
			if ((!empty($settings['login']) && !empty($settings['password'])) || !empty($settings['api_key'])) {
				$answer = AlphasmsSMSApi::alphasms()->getBalance($settings['login'], $settings['password'], $settings['api_key']);

				if (empty($answer['error'])) {
					$balance = AlphasmsSMSApi::alphasms()->getFormat($answer['balance'], 'money');
				}
			}
		}
		
		echo $balance;
		wp_die(); 
	}
	
	/**
	 * select_template() 
	 */
	public function select_template() 
	{
		if (!empty((int) $_GET['id_admin'])) {
			$this->settings['id_admin'] = (int) $_GET['id_admin'];
		}
		
		if (!empty((int) $_GET['id_client'])) {
			$this->settings['id_client'] = (int) $_GET['id_client'];
		}

		$this->settings = $this->settings + AlphasmsDB::alphasms()->loadSettings('sms');
		
		$admin_settings = $this->settings['admin_template_settings'][$this->settings['id_admin']];
		if (!empty($admin_settings) && is_array($admin_settings)) {
			foreach ($admin_settings as $name=>$value) {
				$this->settings[$name] = $value;
			}
		}
		
		$client_settings = $this->settings['client_template_settings'][$this->settings['id_client']];
		if (!empty($client_settings) && is_array($client_settings)) {
			foreach ($client_settings as $name=>$value) {
				$this->settings[$name] = $value;
			}
		}
		
		require_once realpath(ALPHASMS_DIR) . '/views/sms_page/__sms_templates.php';
		wp_die(); 
	}
	
	/**
	 * load_form_smssettings() 
	 */
	public function load_form_smssettings() 
	{
		if (!self::validateToken($_POST['data']['token'])) {
			wp_die(); 
		}
		
		if (!empty($_POST['data']['AlphasmsSmsSettings'])) {
			
			$data = new AlphasmsDB;
			$data->attributes($_POST['data']['AlphasmsSmsSettings']);

			if (!$data->add_data()) {
				$this->settings['error'] = self::addError(__('Not Saved', 'alphasms'));
			} else {
				$this->settings['success'] =__('ConnectSuccess', 'alphasms');	
			} 

		} else {
			$this->settings['error'] =__('Missing data', 'alphasms');
		}

		$this->settings = $this->settings + AlphasmsDB::alphasms()->loadSettings('sms');

		require_once realpath(ALPHASMS_DIR) . '/views/sms_page/__sms_settings.php'; 
		wp_die();
	}

	/**
	 * load_form_smsevents() 
	 */
	public function load_form_smsevents() 
	{
		if (!self::validateToken($_POST['data']['token'])) {
			wp_die(); 
		}

		if (!empty($_POST['data']['AlphasmsSmsEvents'])) {
			
			$data = new AlphasmsDB;
			$data->attributes($_POST['data']['AlphasmsSmsEvents']);

			if (!$data->add_data()) {
				$this->settings['error'] = self::addError(__('No Save', 'alphasms'));
			} else {
				$this->settings['success'] =__('ConnectSuccess', 'alphasms');
			} 	
		} else {
			$this->settings['error'] =__('Missing data', 'alphasms');
		}

		$this->settings = $this->settings + AlphasmsDB::alphasms()->loadSettings('sms');

		require_once realpath(ALPHASMS_DIR) . '/views/sms_page/__sms_events.php'; 
		wp_die(); 
	}
	
	/**
	 * load_form_smstemplates() 
	 */
	public function load_form_smstemplates() 
	{
		if (!self::validateToken($_POST['data']['token'])) {
			wp_die(); 
		}

		if (!empty((int) $_POST['data']['id_admin'])) {
			$this->settings['id_admin'] = (int) $_POST['data']['id_admin'];
		}

		if (!empty((int) $_POST['data']['id_client'])) {
			$this->settings['id_client'] = (int) $_POST['data']['id_client'];
		}

		if (!empty($_POST['data']['AlphasmsSmsTemplates'])) {

			$data = new AlphasmsDB;
			$data->attributes($_POST['data']['AlphasmsSmsTemplates']);

			if (!$data->add_serialize_data()) {
				$this->settings['error'] = self::addError(__('No Save', 'alphasms'));
			} else {
				$this->settings['success'] =__('ConnectSuccess', 'alphasms');
			}	
		} else {
			$this->settings['error'] =__('Missing data', 'alphasms');
		}
		
		$this->settings = $this->settings + AlphasmsDB::alphasms()->loadSettings('sms');
		
		require_once realpath(ALPHASMS_DIR) . '/views/sms_page/__sms_templates.php'; 
		wp_die();
	}

	/**
	 * load_form_smsmailing() 
	 */
	public function load_form_smsmailing() 
	{
		if (!self::validateToken($_POST['data']['token'])) {
			wp_die(); 
		}
		
		$this->settings = $this->settings + AlphasmsDB::alphasms()->loadSettings('sms');

		if (!empty($_POST['data']['AlphasmsSmsMailing'])) {

			if ($_POST['data']['query_type'] === 'send') {

				$data = new AlphasmsSMSApi;
				$data->attributes($_POST['data']['AlphasmsSmsMailing']);
				$array = [
					'text_sms' => empty($data->text_sms) ? '' : $data->text_sms,
					'date_send' => empty($data->date_send) ? date('Y-m-d H:i:s') : $data->date_send,
					'translit' => empty($data->translit) ? 0 : 1,
					'recipient' => empty($data->selection_recipient) ? null : $data->selection_recipient,
				];
				
				$data = self::prepareData($array);

				if (empty($data)) {
					self::addError(__('Missing data', 'alphasms'));
				} else {
					
					$send = AlphasmsSMSApi::alphasms()->send($data);

					if (empty($send['error']) && empty($this->settings['error'])) {
						$this->settings['mailing_settings'] = false;
						$this->settings['success'] =__('Message sent successfully', 'alphasms');
					} else {
						if (empty($this->settings['error'])) {
							$this->settings['error'] = self::addError(__('Could not send SMS', 'alphasms'));
						}
					}
				}
	
			}
			else {
				$this->settings['error'] = self::addError(__('Missing data', 'alphasms'));
			}

		} else {
			$this->settings['error'] =__('Missing data', 'alphasms');
		}

		require_once realpath(ALPHASMS_DIR) . '/views/sms_page/__sms_mailing.php'; 
		wp_die();
	}

	/**
	 * addError($error)
	 */
	public function addError($error='')
	{
		if (!empty($error)) {

			if (empty($this->settings['error'])) {
				
				$this->settings['error'] = $error.'<br>';

			} else {
		
				if (!preg_match('/('.$error.')/i', $this->settings['error'])) {
					$this->settings['error'] .= $error.'<br>';
				}
			}
		}
		
		return $this->settings['error'];
	}
	
	/**
	 * resourceRegistration()
	 */
	public function resourceRegistration() {
	
		wp_enqueue_style('alphasms_bootstrap_min_css', plugins_url('assets/css/bootstrap-3.3.7.min.css', dirname(__FILE__)));
		wp_enqueue_style('alphasms_bootstrap_switch_min_css', plugins_url('assets/css/bootstrap-switch-3.3.2.min.css', dirname(__FILE__)));
		wp_enqueue_style('alphasms_fontawesom_min_css', plugins_url('assets/css/font-awesome-4.7.0.min.css', dirname(__FILE__)));
        wp_enqueue_style('alphasms_plugin_css', plugins_url('assets/css/style.css', dirname(__FILE__)));

		wp_enqueue_script('alphasms_bootstrap_min_js', plugins_url('assets/js/bootstrap3.3.7.min.js', dirname(__FILE__)));
		wp_enqueue_script('alphasms_bootstrap_switch_min_js', plugins_url('assets/js/bootstrap-switch-3.3.2.min.js', dirname(__FILE__)));
		wp_enqueue_script('alphasms_maskedinput_min_js', plugins_url('assets/js/jquery.maskedinput-1.4.1.min.js', dirname(__FILE__)));
        wp_enqueue_script('alphasms_plugin_js', plugins_url('assets/js/script.js', dirname(__FILE__)));
	}
	
	/**
	 * prepareData($array=[])
	 */
	private function prepareData(array $array = [])
	{
		$data = [
//			'login' => $this->settings['login'],
//			'password' => $this->settings['password'],
//			'api_key' => $this->settings['api_key'],
//			'alphaname' => $this->settings['alphaname'],
//			'admin_phone' => $this->settings['admin_phone'],
//			'admin_phone2' => $this->settings['admin_phone2'],
//			'admin_phone3' => $this->settings['admin_phone3'],
            'message' => [],
		];

		if (empty($this->settings['alphaname'])) {
			return false;
		}

//		if (empty($array['order']) && !is_array($array['order'])) {
//			return false;
//		}

		$stores_name = get_bloginfo();
		$replace['StoresName'] = !empty($stores_name) ? $stores_name : '';
			
		$inc = 0;

		if (empty($array['order'])) {

            if (!empty($array['text_sms'])) {
                $data['message'][$inc] = [
                    'text_sms' => trim($array['text_sms']),
                    'recipient' => AlphasmsSMSApi::alphasms()->getFormat($array['recipient'], 'phone'),
                ];
            }

		    return $data;
        }

		foreach ($array['order'] as $key=>$value) {

            $replace['OrderID'] = !empty($value['order_id']) ? $value['order_id'] : '';
            $replace['OrderSum'] = (!empty($value['_order_total']) && !empty($value['_order_currency'])) ? $value['_order_total'] . ' ' . $value['_order_currency'] : '';
            $replace['ClientName'] = !empty($value['_billing_first_name']) ? $value['_billing_first_name'] : '';
            $replace['ClientLastName'] = !empty($value['_billing_last_name']) ? $value['_billing_last_name'] : '';
            $replace['OrderStatus'] = !empty($value['status']) ? __($this->settings['status'][$value['status']], 'alphasms') : '';

            $replace['AddrOrderDelivery'] = '';
            if (!empty($value['_shipping_city'])) {
                $replace['AddrOrderDelivery'] .= $value['_shipping_city'] . ' ';
            }

            if (!empty($value['_shipping_address_2'])) {
                $replace['AddrOrderDelivery'] .= $value['_shipping_address_2'] . ' ';
            }

            if (!empty($value['_shipping_address_1'])) {
                $replace['AddrOrderDelivery'] .= $value['_shipping_address_1'] . ' ';
            }

            $replace['AddrPayment'] = '';
            if (!empty($value['_billing_city'])) {
                $replace['AddrPayment'] .= $value['_billing_city'] . ' ';
            }

            if (!empty($value['_billing_address_2'])) {
                $replace['AddrPayment'] .= $value['_billing_address_2'] . ' ';
            }

            if (!empty($value['_billing_address_1'])) {
                $replace['AddrPayment'] .= $value['_billing_address_1'] . ' ';
            }

            $replace['MethodPayment'] = !empty($value['_payment_method_title']) ? $value['_payment_method_title'] : '';

            $message_arr = [
                'text_sms' => AlphasmsSMSApi::alphasms()->replaceTag($array['template_mailing'], $replace),
                'recipient' => AlphasmsSMSApi::alphasms()->getFormat($key, 'phone'),
            ];

            if (!empty($array['translit'])) {
                $message_arr['sms_text'] = AlphasmsSMSApi::alphasms()->transliterate($message_arr['sms_text']);
            }

            if (!empty($array['date_send'])) {
                $message_arr['date_beg'] = str_replace(' ', 'T', $array['date_send']);
            }

            $data['message'][$inc] = $message_arr;

            $inc++;
        }

		return $data;	
	}

//	private function getStatusArray()
//	{
//		$array_status = [];
//		if (!empty($_POST['data']['AlphasmsSmsMailing']) && is_array($_POST['data']['AlphasmsSmsMailing'])) {
//
//			foreach ($_POST['data']['AlphasmsSmsMailing'] as $name=>$value) {
//
//				if (preg_match('/^([a-z]{1,})+(_recipients)$/i', $name)) {
//
//					if (AlphasmsValidate::validateBoolean($value)) {
//
//						$status = preg_replace('/(_recipients)/i', '', $name);
//						if ($status=='onhold') {
//							$status = preg_replace('/^(on).*?/i', '$1-', $name);
//						}
//
//						$array_status[] = $status;
//						$this->settings['mailing_settings'][$name] = 1;
//					} else {
//						$this->settings['mailing_settings'][$name] = 0;
//					}
//				}
//			}
//		}
//
//		return $array_status;
//	}
	
	/**
	 * validateToken()
	 */
	private function validateToken($str='')
	{
		$pattern = '/[a-z 0-9 \%\/]/i';
		if (
			!empty($str) && 
			preg_match($pattern, $str) && 
			$str===wp_get_session_token()
		) {
			return true;
		}

		return false;
	}
	
	/**
	 * install()
	 */
	public static function install()
	{
		/* Detect WooCommerce */
		if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))){
			die(__('Error Install', 'alphasms'));
		}
		
		AlphasmsDB::alphasms()->create_table();
	}
	
	/**
	 * uninstall()
	 */
	public static function uninstall()
	{
		AlphasmsDB::alphasms()->delete_table();
	}
	
	/**
	 * deactivation()
	 */
	public static function deactivation()
	{
		AlphasmsDB::alphasms()->truncate_table();
	}
	
	/**
	 * alphasms($is_plugin_page, $className=__CLASS__)
	 */ 
	public static function alphasms($is_plugin_page = null, $className=__CLASS__)
	{
		return new $className($is_plugin_page);
	}
}
