<?php 
class AlphasmsSMSApi
{	
	/**
	 * History
	 */
	public $date_start;
	public $date_stop;
	public $state = 'deliver';
	public $originator = 'target';
	public $phone;
	public $operator;
	public $from_hour = '00';
	public $from_minute = '00';
	public $to_hour = '23';
	public $to_minute = '59';
	
	/**
	 * Send
	 */
	public $text_sms;
	public $used_translit;
	public $all_recipients;
	public $pending_recipients;
	public $processing_recipients;
	public $onhold_recipients;
	public $completed_recipients;
	public $canceled_recipients;
	public $refunded_recipients;
	public $failed_recipients;
	public $selection_recipient;
	public $date_send;

	/**
	 * config()
	 */
	public function config()
	{
		$config_path = realpath(ALPHASMS_DIR) . '/config.php';
		if (file_exists($config_path)) {
			$config = require($config_path);
			
			return $config;
		}
	}
	
	/**
	 * rules() 
	 */
	public function rules() 
	{
		return [
			['date_start, date_stop, date_send', 'Pattern', '/\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}/i'],
			['state, originator, operator', 'Pattern', '/^[a-z]{1,20}$/i'],
			['from_hour, from_minute, to_hour, to_minute', 'Pattern', '/^[0-9]{2}$/i'],
			['selection_recipient', 'Pattern', '/^[0-9a-z \+\-\(\)\s\,]{1,}$/i'],
			['phone', 'function', 'cleaningNumber'],
			['used_translit, all_recipients, pending_recipients, processing_recipients, on_hold_recipients, completed_recipients, canceled_recipients, refunded_recipients, failed_recipients', 'Boolean'],
			['text_sms', 'function', 'cleaningTextSms'],
		];
	}
	
	/**
	 * attributes($array=[]) 
	 */
	public function attributes($array=[]) 
	{
		if (!empty($array)) {
			
			$rules = self::rules();
			if (empty($rules) || !is_array($rules)) {
				return false;
			}

			foreach ($array as $key=>$value) {
				
				if (empty($key) || !is_string($key) || !property_exists($this, $key)) {
					continue;
				}

				foreach ($rules as $rule) {

					if (empty($rule[0])) {
						continue;
					}

					$rule[0] = preg_replace('/[\s\r\n]/i', '', $rule[0]);
					$properties = explode(',', $rule[0]);
					
					if (
						empty($properties) || 
						!is_array($properties) || 
						empty($rule[1]) ||
						!is_string($rule[1])
					) {
						continue;
					}

					if (in_array($key, $properties)) {

						if (
							$rule[1]==='function' && 
							!empty($rule[2]) && 
							is_string($rule[2]) && 
							method_exists($this, $rule[2])
						) {
									
							$method = $rule[2];
							$this->$key = $this->$method($value);
		
						} elseif (method_exists('AlphasmsValidate', 'validate'.$rule[1])) {
			
							$method = 'validate'.$rule[1];
							$param = '';
							if (!empty($rule[2]) && is_string($rule[2])) {
								$param = $rule[2];
							}
							
							if (AlphasmsValidate::$method($value, $param)) {

								$this->$key = $value;

							} else {
									
								$this->$key = false;
									
							}					
						}	

						break;
					}
				}
			}
		}
	}
	
	/**
	 * getBalance($login='', $password='')
	 */
	public function getBalance($login='', $password='', $api_key = null)
	{
	    $xml = '<?xml version="1.0" encoding="utf-8" ?>';
		if (!empty($login) && !empty($password)) {
            $xml .= '<package login="'.$login.'" password="'.$password.'"><balance></balance></package>';
        }
        else if (!empty($api_key)) {
            $xml .= '<package key="'.$api_key.'"><balance></balance></package>';
        }

		$answer_xml = self::request($xml, $this->config()['sms_api_url']);

		if (empty($answer_xml)) {
			return ['error'=>__('No Answer', 'alphasms')];
		}
		
		$data = self::parseXML($answer_xml);

		if (empty($data->balance->amount)) {
			if (!empty($data['error'])) {
				return ['error'=>$data['error']];
			} else {
				return ['error'=>__('No Answer', 'alphasms')];
			}
		}

		return [
		    'error' => 0,
            'balance' => strval($data->balance->amount),
            'currency' => strval($data->balance->currency),
        ];
	}

	/**
	 * send($data=[])
	 */
	public function send($data=[])
	{
		$settings = AlphasmsDB::alphasms()->loadSettings('sms');

		if (((empty($settings['login']) || empty($settings['password'])) && empty($settings['api_key'])) || empty($settings['alphaname'])) {
			return false;
		}	

		if (empty($data['message']) || !is_array($data['message'])) {
			return false;
		}

        $xml_msg = '';

        foreach ($data['message'] as $n => $message_arr) {

            if (empty($message_arr['recipient']) || empty($message_arr['text_sms'])) {
                continue;
            }

            $recipient = preg_replace('/\D/', '', $message_arr['recipient']);

            $xml_msg .= '<msg recipient="' . $recipient  . '" sender="' . $settings['alphaname'] . '" type="0"' .
                (empty($data['date_beg']) ? '' : ' date_beg="'.$data['date_beg'].'"' ) . '>' . $message_arr['text_sms'] . '</msg>';
        }


        if (!empty($xml_msg)) {

            $xml = '<?xml version="1.0" encoding="utf-8" ?>';
            if (!empty($settings['login']) && !empty($settings['password'])) {
                $xml .= '<package login="' . $settings['login'] . '" password="' . $settings['password'] . '">';
            }
            else if (!empty($settings['api_key'])) {
                $xml .= '<package key="' . $settings['api_key'] . '">';
            }
            else {
                $xml .= '<package>';
            }
            $xml .= '<message>';
            $xml .= $xml_msg;
            $xml .= '</message>';
            $xml .= '</package>';

            $answer_xml = self::request($xml, $this->config()['sms_api_url']);
        }

		if (empty($answer_xml)) {
			return ['error'=>__('No Answer', 'alphasms')];
		}

		$results = self::parseXML($answer_xml);

		if (empty($results->message)) {
			if (!empty($results->error)) {
				return ['error'=>$results->error];
			} else {
				return ['error'=>__('No Answer', 'alphasms')];
			}
		}

		return ['error' => 0, 'send' => 1];
	}

	/**
	 * replaceTag($str='', $data=[])
	 */
	public function replaceTag($str='', $data=[])
	{
		if (empty($data) || empty($this->config()['tag'])) {
			return $str;
		}

		foreach ($this->config()['tag'] as $key=>$value) {
			$search[] = '{'.$value.'}';
			$replace[] = empty($data[$key]) ? '' : $data[$key];
		}

		return str_replace($search, $replace, $str);
	}
	
	/**
	 * transliterate($str)
	 */
	public function transliterate($str) 
	{
		$for_replacement = [
            'А' => 'A',   'а' => 'a',
            'Б' => 'B',   'б' => 'b',
            'В' => 'V',   'в' => 'v',
            'Г' => 'G',   'г' => 'g',
            'Д' => 'D',   'д' => 'd',
            'Е' => 'E',   'е' => 'e', 
            'Ё' => 'YO',  'ё' => 'yo',
            'Ж' => 'ZH',  'ж' => 'zh',
            'З' => 'Z',   'з' => 'z',
            'И' => 'I',   'и' => 'i', 
            'Й' => 'J',   'й' => 'j', 
            'К' => 'K',   'к' => 'k',
            'Л' => 'L',   'л' => 'l',
            'М' => 'M',   'м' => 'm',
            'Н' => 'N',   'н' => 'n', 
            'О' => 'O',   'о' => 'o', 
            'П' => 'P',   'п' => 'p', 
            'Р' => 'R',   'р' => 'r',
            'С' => 'S',   'с' => 's',
            'Т' => 'T',   'т' => 't', 
            'У' => 'U',   'у' => 'u', 
            'Ф' => 'F',   'ф' => 'f', 
            'Х' => 'H',   'х' => 'h', 
            'Ц' => 'C',   'ц' => 'c', 
            'Ч' => 'CH',  'ч' => 'ch', 
            'Ш' => 'SH',  'ш' => 'sh', 
            'Щ' => 'SCH', 'щ' => 'sch', 
            'Ь' => '',    'ь' => '', 
            'Ы' => 'Y',   'ы' => 'y', 
            'Ъ' => '',    'ъ' => '', 
            'Э' => 'E',   'э' => 'e', 
            'Ю' => 'YU',  'ю' => 'yu', 
            'Я' => 'YA',  'я' => 'ya',            
		]; 
 
		$output = str_replace( 
			array_keys($for_replacement), 
			array_values($for_replacement), $str 
		); 
 
		return $output; 
	}
	
	/**
	 * getFormat($float)
	 */
	public static function getFormat($int=0, $format=0)
	{
		if (empty($int)) {
			return 0;
		}
		
		if (empty($format)) {
			return $int;
		} 
		
		if ($format=='money') {
			
			return @number_format($int, 2, '.', ' ');
			
		} elseif ($format=='number') {
			
			return @number_format($int, 0, ',', ' ');
			
		} elseif ($format=='float') {
			
			return @number_format($int, 2, ',', ' ');
			
		} elseif ($format=='phone') {
			
			return preg_replace('/\D/i', '', $int);

		} else {
			return $int;
		}
	}


	/** 
	 * parseXML($xml)
	 */
	private function parseXML($xml)
	{
        try {
            $data = new \SimpleXMLElement($xml, LIBXML_NOERROR | LIBXML_NOWARNING | LIBXML_NOCDATA);
        } catch (\Exception $e) {
            exit($e->getMessage());
        }


//		$p = xml_parser_create();
//		xml_parse_into_struct($p, $xml, $vals, $index);
//
//		$data = [];
//		foreach ($vals as $value) {
//
//			if ($value['tag']=='RESPONSE') {
//				continue;
//			}
//
//			if ($value['tag']=='SMS') {
//
//				$data['sms'][] = [
//					'area' => $value['attributes']['AREA'],
//					'value' => $value['value'],
//				];
//
//			}
//
//			if ($value['tag']=='MONEY') {
//
//				$data['balance'][] = [
//					'currency' => $value['attributes']['CURRENCY'],
//					'value' => $value['value'],
//				];
//
//			}
//
//			if ($value['tag']=='ORIGINATOR') {
//
//				if ($value['attributes']['STATE']!=='completed') {
//					continue;
//				}
//
//				$data['from_name'][] = [
//					'value' => $value['value'],
//				];
//
//			}
//
//			if ($value['tag']=='PATTERN') {
//
//				$data['templates'][$value['attributes']['ID_PATTERN']] = [
//					'value' => $value['value'],
//					'name' => $value['attributes']['NAME'],
//				];
//
//			}
//
//			if ($value['tag']=='INFORMATION') {
//				$data['send'][] = [
//					'attributes' => [
//						'num' => $value['attributes']['NUMBER_SMS'],
//						'id' => $value['attributes']['ID_SMS'],
//						'turn' => $value['attributes']['ID_TURN'],
//						'parts' => $value['attributes']['PARTS'],
//					],
//					'value' => $value['value'],
//				];
//			}
//
//			if ($value['tag']=='ERROR') {
//
//				$data['error'] = $value['value'];
//
//			}
//
//			if ($value['tag']=='STATS') {
//
//				if (!empty($value['attributes']) && $value['attributes']['NUM_STATS']==0) {
//					return $data;
//				}
//
//			}
//
//			if ($value['tag']=='STAT') {
//				$data['stat'][] = [
//					'id' => $value['attributes']['ID_SMS'],
//                    'state' => $value['attributes']['ID_STATE'],
//                    'operator' => $value['attributes']['OPERATOR'],
//                    'delivery' => $value['attributes']['NAME_DELIVERY'],
//                    'phone' => $value['attributes']['PHONE'],
//                    'originator' => $value['attributes']['ORIGINATOR'],
//                    'time_state' => $value['attributes']['TIME_CHANGE_STATE'],
//                    'time' => $value['attributes']['TIME'],
//                    'status'=> $value['attributes']['STATUS'],
//                    'status_title' => $value['attributes']['STATUS_TRANSLATE'],
//                    'text' => $value['attributes']['TEXT'],
//                    'price' => $value['attributes']['PRICE'],
//                    'parts' => $value['attributes']['NUM_PARTS'],
//                    'partno' => $value['attributes']['PART_NO'],
//				];
//			}
//		}
		
		return empty($data['package']) ? $data : $data['package'];
	}
	
	/**
	 * cleaningTextSms()
	 */
	private function cleaningTextSms($str='')
	{
		if (empty($str) || !is_string($str)) {
			return '';
		}

		$str = strip_tags($str);
		$str = htmlspecialchars($str);
		$str = htmlentities($str);
		
		return $str;
	}
	
	/**
	 * cleaningNumberArray($array=[])
	 */
	private function cleaningNumber($number='')
	{
		if (preg_match('/^[0-9 \+\-\(\)]{1,}$/i', $number)) {
				
			$number = preg_replace('/[\+\-\(\)\s]/i', '', $number);
			if (strlen($number)>15 || strlen($number)<9) {
				return false;
			}

		} elseif (preg_match('/^[0-9a-z \+\-\(\)]{1,}$/i', $number)) {	
				
			$number = preg_replace('/[\+\-\(\)\s]/i', '', $number);
			if (strlen($number)>11 || strlen($number)<5) {
				return false;
			}
				
		} else {
			return false;
		}

		return $number;
	}

	/**
	 * request($xml, $url)
	 */
	private function request($xml, $url)
	{
		$response = wp_remote_post($url, [
			'method'      => 'POST',
			'headers'     => 'Content-type: text/xml; charset=utf-8',
			'body'        => $xml
		]);

		if (is_wp_error($response)) {
			return ['error'=>$response->get_error_message()];
		} else {
			$result = $response['body'];
		}
		
		return $result;
	}

	/**
	 * alphasms($className=__CLASS__)
	 */ 
	public static function alphasms($className = __CLASS__)
	{
		return new $className();
	}
}
